/**
 ** sahandFunctions.cc
 **
 ** Utilitarian functions for codels of sahand module
 **
 ** Author: Daniel Sidobre
 ** Date: 12 mai 2009
 **
 **/

#include <portLib.h>
#include <h2mathLib.h>

#include "server/sahandHeader.h"
#include "sahandConst.h"
#include "sahandPrivate.h"

#define SAH_MIN_THUMB_ORIENTATION (0.0)
#define SAH_MAX_THUMB_ORIENTATION (DEG_TO_RAD(90.0))

double sah_min_position[3] = { 0.0, 0.0, 0.0 };
double sah_max_position[3] = { 0.0, 0.0, 0.0 };
  
double sah_max_velocity[3] = { 0.0, 0.0, 0.0 };

double sah_max_stiffness[3] = { 0.0, 0.0, 0.0 };

/*
 * sahandInitCheck() : init sah_min/max_... values.
 */
void sahandInitCheck()
{
  sah_min_position[SAHAND_DISTAL]   = SAH_MIN_DISTAL_ANGLE;
  sah_min_position[SAHAND_PROXIMAL] = SAH_MIN_PROXIMAL_ANGLE;
  sah_min_position[SAHAND_BASE]     = SAH_MIN_BASE_ANGLE;

  sah_max_position[SAHAND_DISTAL]   = SAH_MAX_DISTAL_ANGLE;
  sah_max_position[SAHAND_PROXIMAL] = SAH_MAX_PROXIMAL_ANGLE;
  sah_max_position[SAHAND_BASE]     = SAH_MAX_BASE_ANGLE;

  sah_max_velocity[SAHAND_DISTAL]   = SAH_MAX_DISTAL_VEL;
  sah_max_velocity[SAHAND_PROXIMAL] = SAH_MAX_PROXIMAL_VEL;
  sah_max_velocity[SAHAND_BASE]     = SAH_MAX_BASE_VEL;

  sah_max_stiffness[SAHAND_DISTAL]   = SAH_MAX_DISTAL_STIFFNESS;  
  sah_max_stiffness[SAHAND_PROXIMAL] = SAH_MAX_PROXIMAL_STIFFNESS;
  sah_max_stiffness[SAHAND_BASE]     = SAH_MAX_BASE_STIFFNESS;
}

/*
 * sahandFingerPositionLimit
 * Limit position parameters for the finger in the admissible range.
 *
 * Return the number of value limited
 */
int sahandFingerPositionLimit (double* val)
{
  int i;
  int retval = 0;
  for (i = 0; i < 3; i++) {
    if (val[i] < sah_min_position[i]) {
      val[i] = sah_min_position[i];
      retval++;
    }
    if (val[i] > sah_max_position[i]) {
      val[i] > sah_max_position[i];
      retval++;
    }
  }
  return retval;
}

/*
 * sahandFingerPositionCheck
 *
 * return :
 *  0 if OK
 *  -1 if one val if inferior to min
 *   1 if one val is superior to max
 */
int sahandFingerPositionCheck (double* val)
{
  int i;

  for (i = 0; i < 3; i++) {
    if (val[i] < sah_min_position[i]) {
      printf("SAH bad position: %d goal = %f min = %f\n",i, val[i],  sah_min_position[i]);
      return -1;
    }
    if (val[i] > sah_max_position[i]){
      printf("SAH bad position: %d goal = %f max = %f\n",i, val[i],  sah_max_position[i]);
      return 1;
    }
  }
  return 0;
}

/*
 * sahandFingerVelocityCheck
 *
 * return :
 *  0 if OK
 *  -1 if one val if inferior to 0
 *   1 if one val is superior to max
 */
int sahandFingerVelocityCheck (double* val)
{
  int i;

  for (i = 0; i < 3; i++) {
    if (val[i] < 0){
      printf("SAH bad velocity: %d goal = %f min = %f\n",i, val[i], 0);
      return -1;
    }
    if (val[i] > sah_max_velocity[i]){
      printf("SAH bad velocity: %d goal = %f max = %f\n",i, val[i], sah_max_velocity[i]);
      return 1;
    }
  }
  return 0;
}

/*
 * sahandFingerStiffnessCheck
 *
 * return :
 *  0 if OK
 *  -1 if one val if inferior to 0
 *   1 if one val is superior to max
 */
int sahandFingerStiffnessCheck (double* val)
{
  int i;

  for (i = 0; i < 3; i++) {
    if (val[i] < 0) return -1;
    if (val[i] > sah_max_stiffness[i]) return 1;
  }
  return 0;
}


